#include "menu.h"
#include "config.h"
#include "gfxinterface.h"
#include "input.h"
#include "pix.h"
#include "time_util.h"
#include "xpms.h"


const char *const menu_lines[] = {
	"XGalaga++",
	"v0.4 (2003-10-15)",
	"",
	"2003 Marc Mongenet, graphics from XGalaga 2.0.34",
	"This game is free software, covered by the GPL.",
	"",
	"Key             Action",
	"             ",
	"s               start",
	"q               quit",
	"p               pause",
	"1,2,3,4         set window size",
	"+               increase details (slower)",
	"-               decrease details (faster)",
	"<space bar>     fire",
	"<left arrow>    move left",
	"<right arrow>   move right",
	"",
	"Bonus",
	"",
	"extra shield every 1000 points"
};


StartMenu * StartMenu::singleton_ (0);


StartMenu::StartMenu()
: font_(X11::Inst().LoadQueryFont("*-fixed-bold-r-*-*-14-*-*-*-*-*-iso8859-1"))
, width_(0)
, green_comp_(0)
, green_add_(1024)
{
	if (!font_) {
		font_ = X11::Inst().LoadQueryFont("*-fixed-*");
		if (!font_) throw std::runtime_error ("X font error");
	}

	for (size_t i (0); i < sizeof menu_lines / sizeof menu_lines[0]; ++i) {
		width_ = std::max(width_, XTextWidth(font_, menu_lines[i],
		                                     std::strlen(menu_lines[i])));
	}
}


StartMenu::~StartMenu()
{
	X11::Inst().FreeFont(font_);
}


void SetWindowSize(unsigned win_size)
{
	static const unsigned widths[]  = { 640, 800, 1024, 1280, 1600 };
	static const unsigned heights[] = { 480, 600, 768, 1024, 1200 };
	if (win_size > 0 && win_size <= sizeof widths / sizeof widths[0]) {
		--win_size;
		X11::Inst().ResizeWindow(widths[win_size], heights[win_size]);
		X11::Inst().ClearWindow();
	}
}


bool StartMenu::Display(std::vector<StarsField> * stars_fields, Score * score)
{
	Input input;
        for (;;) {
		const double start_frame_time (CurrentMicroSecond());
		input.Update();
		if (input.Quit()) return false;
		if (input.Start()) return true;
		SetWindowSize(input.WindowSize());
		Details::Instance().Change(input.Details());
		X11::Inst().GetWindowAttributes();
		ScrollStarFields(stars_fields);
		score->Refresh();
		Print();
		X11::Inst().Sync(False);
		WaitNextFrame(start_frame_time);
	}
}


void StartMenu::Print()
{
	const int left (std::max(0, (X11::Inst().WindowWidth() - width_) / 2));
	Coord pos (left, 20);
	X11::Inst().SetClipMask(None);
	X11::Inst().SetFont(font_->fid);
	green_comp_ += green_add_;
	if (green_comp_ == 0) {
		green_add_ = - green_add_;
		green_comp_ += green_add_;
	}
	XColor color;
	color.red = 48*1024;
	color.green = green_comp_;
	color.blue = 65535;
	for (size_t i (0); i < sizeof menu_lines / sizeof menu_lines[0]; ++i) {
		X11::Inst().AllocColorAlways(&color);
		X11::Inst().SetForeground(color.pixel);
		X11::Inst().DrawString(pos, menu_lines[i]);
		pos.y += font_->ascent + font_->descent + 1;
		if (!menu_lines[i][0]) {
			color.red -= 8192;
		}
	}

	static const char **const bonuses[] = {
		bonus_speed_xpm, bonus_fire_xpm, bonus_shield_xpm
	};
	static const char *const bonus_text[] = {
		"extra speed", "extra fire", "extra shield"
	};
	for (size_t i (0); i < sizeof bonuses / sizeof bonuses[0]; ++i) {
		const Pix *const bonus (PixKeeper::Instance().Get(bonuses[i]));
		bonus->Draw(pos + Coord(bonus->Width()/2, -bonus->Height()/2));
		X11::Inst().
		X11::Inst().SetClipMask(None);
		X11::Inst().DrawString(pos + Coord(2 * bonus->Width(), 0),
		                       bonus_text[i]);
		pos.y += std::max(bonus->Height(),
		                  font_->ascent + font_->descent + 1);
	}
}
